import { join } from '../fn.js'
import { alpha, alphanumeric, any, char, digit, list, many, map, maybe, noCaseString, separated, seq, skip, string, until } from '../parser.js'

export const Symbol = Object.freeze({
  Bracket: Object.freeze({
    Angle: Object.freeze({
      Left: char('<'),
      Right: char('>')
    }),
    Curly: Object.freeze({
      Left: char('{'),
      Right: char('}'),
    }),
    Round: Object.freeze({
      Left: char('('),
      Right: char(')'),
    }),
    Square: Object.freeze({
      Left: char('['),
      Right: char(']'),
    }),
  }),
  Colon: char(':'),
  Comma: char(','),
  Hyphen: char('-'),
  Newline: char('\n'),
  Period: char('.'),
  Plus: char('+'),
  Space: char(' '),
  Quote: char('"')
})



export const collect = parser => map(join(''), parser)
export const quoted = collect(seq(skip(Symbol.Quote), until(Symbol.Quote), skip(Symbol.Quote)))
export const number = seq(digit, many(digit))
export const signed = seq(maybe(any(Symbol.Plus, Symbol.Hyphen)), number)
export const toBoolean = v => v === 'false' ? false : true

export const Literal = Object.freeze({
  Float: map(([n]) => parseFloat(n, 10), collect(seq(signed, Symbol.Period, number))),
  Integer: map(([n]) => parseInt(n, 10), collect(signed)),
  String: quoted,
  Boolean: map(([v]) => toBoolean(v), any(string('true'), string('false')))
})

export const literal = any(...Object.values(Literal))
export const ws = skip(many(Symbol.Space))
export const identifier = map(join(''), seq(alpha, many(alphanumeric)))
export const accessor = seq(identifier, list(Symbol.Period, identifier))
